@@ -1,6 +1,6 @@
 /*
 Created		21/02/2003
-Modified		7/01/2004
+Modified		10/01/2004
 Project		tcap
 Model		core
 Company		The Company in Mianos
@@ -52,7 +52,7 @@ Create table "usage"
 	"last_seen" Timestamp NOT NULL,
 	"packets" integer NOT NULL,
 	"volume" Bigint NOT NULL,
-	"flushed" integer,
+	"flushed_at" Timestamp,
  primary key ("src_ip_serial","port_src","dst_ip_serial","port_dst","protocol_number","interface","first_seen")
 );
 
