@@ -65,6 +65,7 @@ int	DnsCache::FindHostFromIP(const struct in_addr& ip, const char *mac) {
 	int	res;
 	int	err;
 	string	resolvedName;
+	char	inet_char[inet_to_stringSize];
 
 	memset(buf, 0, sizeof (buf));
 	memset(&hpb, 0, sizeof (hpb));
@@ -74,9 +75,10 @@ int	DnsCache::FindHostFromIP(const struct in_addr& ip, const char *mac) {
 								 &hpb,
 								 buf, sizeof(buf),
 								 &hp, &err);
+	inet_to_char(ip, inet_char);
 	if (errno == 0) {
 		if (hp == 0) {
-			resolvedName = inet_to_string(ip);
+			resolvedName = inet_char;
 			//cerr << "resolved name '" << resolvedName << "' returned null host entry, res was " << res << endl;
 		} else {
 		// convert to lower case
@@ -86,11 +88,11 @@ int	DnsCache::FindHostFromIP(const struct in_addr& ip, const char *mac) {
 			//cerr << "resolved name '" << resolvedName << endl;
 		}
 	} else if (errno == ENOENT) {
-		resolvedName = inet_to_string(ip);
+		resolvedName = inet_char;
 	} else {
 		syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR), "gethostbyaddr_r: %s looking for %s",
-			sys_errlist[errno], inet_to_string(ip).c_str());
-		resolvedName = inet_to_string(ip);
+			sys_errlist[errno], inet_char);
+		resolvedName = inet_char;
 	}
 	// create temporary Dns record for lookup
 	DnsRecord *inDns = new DnsRecord(ip, resolvedName);
@@ -115,3 +117,12 @@ int	DnsCache::FindHostFromIP(const struct in_addr& ip, const char *mac) {
 	return newDns->serial_m;
 }
 
+char *inet_to_char(struct in_addr ip, char *out) {
+	unsigned char *ucp = (unsigned char *)&ip;
+	sprintf(out, "%d.%d.%d.%d",
+		ucp[0] & 0xff,
+		ucp[1] & 0xff,
+		ucp[2] & 0xff,
+		ucp[3] & 0xff);
+	return out;
+}
