@@ -13,7 +13,9 @@ public:
 	DnsRecord(struct in_addr ip, string& hostName, int serial, time_t last_access);
 	DnsRecord(struct in_addr ip, string hostName);
     friend ostream& operator<<(ostream &os, const DnsRecord *cr) {
-		os << "src " << inet_to_string(cr->ip_m)
+		char	inet_char[inet_to_stringSize];
+		inet_to_char(cr->ip_m, inet_char);
+		os << "src " << inet_char
 			<< " host " << cr->hostName_m
 			<< " serial " << cr->serial_m
 			<< " last " << ctime(&cr->last_access_m);
