@@ -18,16 +18,9 @@ struct my_ip {
 };
 
 
-inline string inet_to_string(struct in_addr ip) {
-	char buf[4 + 4 + 4 + 5 + 1];
-	unsigned char *ucp = (unsigned char *)&ip;
-	sprintf(buf, "%d.%d.%d.%d",
-		ucp[0] & 0xff,
-		ucp[1] & 0xff,
-		ucp[2] & 0xff,
-		ucp[3] & 0xff);
-	return string(buf);
-}
+#define	inet_to_stringSize	(4 + 4 + 4 + 5 + 1)
+
+extern char *inet_to_char(struct in_addr ip, char *out);
 
 inline string	port_to_string(u_int16_t port) {
 	char	buf[32];
@@ -42,7 +35,9 @@ inline string	protocol_to_string(u_int8_t proto) {
 }
 
 inline ostream& operator<<(ostream &os, struct in_addr ip) {
-	os << inet_to_string(ip);
+	char	ip_char[inet_to_stringSize];
+	inet_to_char(ip, ip_char);
+	os << ip_char;
 	return os;
 }
 
