@@ -15,9 +15,12 @@
 #include <pthread.h>
 #include <sys/time.h>
 #include <unistd.h>
-#include <pcap.h>
 #include <syslog.h>
 
+extern "C" {
+#include <pcap.h>
+}
+
 #include <unistd.h>
 #include <stdio.h>
 #include <getopt.h>
@@ -107,29 +110,32 @@ struct sqlca_t *ECPGget_sqlca(void);
 
 #endif
 
-#line 33 "sqlwrite.ec"
+#line 36 "sqlwrite.ec"
 
 
+static pthread_mutex_t SqlWriteMutex;
+
 void	SQLConnect(char *connectionInfo)
 {
 	/* exec sql begin declare section */
 		
 	
-#line 38 "sqlwrite.ec"
+#line 43 "sqlwrite.ec"
  char * connection   ;
 /* exec sql end declare section */
-#line 39 "sqlwrite.ec"
+#line 44 "sqlwrite.ec"
 
 
 	connection = connectionInfo;
 
 	{ ECPGconnect(__LINE__, 0, connection , NULL,NULL , NULL, 0); }
-#line 43 "sqlwrite.ec"
+#line 48 "sqlwrite.ec"
 
 	if(sqlca.sqlcode) {
 		cerr << "connect Error # " << sqlca.sqlerrm.sqlerrml << " '" << sqlca.sqlerrm.sqlerrmc << "'" << endl;
 		exit(0);
 	}
+	pthread_mutex_init(&SqlWriteMutex, NULL);
 }
 
 static void timevalToDate(const struct timeval *in, char *out)
@@ -198,7 +204,7 @@ static char	*ToPsqlBinaryString(CDataBlockList& dbl)
 	return "crapola";
 }
 
-bool	SQlSaveRecord(CRecord *cr, string interfaceName)
+bool	SQlSaveRecord(CRecord *cr, string interfaceName,  struct timeval *now)
 {
 	struct in_addr kk;
 	/* exec sql begin declare section */
@@ -217,49 +223,49 @@ bool	SQlSaveRecord(CRecord *cr, string interfaceName)
 			
 		
 	
-#line 120 "sqlwrite.ec"
+#line 126 "sqlwrite.ec"
  int  f_src_ip_serial   ;
  
-#line 121 "sqlwrite.ec"
+#line 127 "sqlwrite.ec"
  int  f_dst_ip_serial   ;
  
-#line 122 "sqlwrite.ec"
+#line 128 "sqlwrite.ec"
  int  f_port_src   ;
  
-#line 123 "sqlwrite.ec"
+#line 129 "sqlwrite.ec"
  int  f_port_dst   ;
  
-#line 124 "sqlwrite.ec"
+#line 130 "sqlwrite.ec"
  int  f_protocol_number   ;
  
-#line 125 "sqlwrite.ec"
+#line 131 "sqlwrite.ec"
  int  f_volume   ;
  
-#line 126 "sqlwrite.ec"
+#line 132 "sqlwrite.ec"
  int  f_packets   ;
  
-#line 127 "sqlwrite.ec"
- int  f_flushed   ;
+#line 133 "sqlwrite.ec"
+ char  f_flushed_at [ 30 ]   ;
  
-#line 128 "sqlwrite.ec"
- int  ind_flushed   ;
+#line 134 "sqlwrite.ec"
+ int  ind_flushed_at   ;
  
-#line 129 "sqlwrite.ec"
+#line 135 "sqlwrite.ec"
  char  f_lastUpdate [ 30 ]   ;
  
-#line 130 "sqlwrite.ec"
+#line 136 "sqlwrite.ec"
  char  f_firstUpdate [ 30 ]   ;
  
-#line 131 "sqlwrite.ec"
+#line 137 "sqlwrite.ec"
  const char * f_interface   ;
  
-#line 132 "sqlwrite.ec"
+#line 138 "sqlwrite.ec"
  int  ind_contents   ;
  
-#line 133 "sqlwrite.ec"
+#line 139 "sqlwrite.ec"
  char * f_contents   ;
 /* exec sql end declare section */
-#line 134 "sqlwrite.ec"
+#line 140 "sqlwrite.ec"
 
 
 	f_port_src = cr->port_src_m;
@@ -269,11 +275,11 @@ bool	SQlSaveRecord(CRecord *cr, string interfaceName)
 	f_protocol_number = cr->ip_proto_m;
 	f_volume = cr->volume_m;
 	f_packets = cr->packets_m;
-	if (cr->flushed_m) {
-		f_flushed = cr->flushed_m;
-		ind_flushed = 0;
+	if (now) {
+		timevalToDate(now, f_flushed_at);
+		ind_flushed_at = 0;
 	} else
-		ind_flushed = -1;
+		ind_flushed_at = -1;
 	timevalToDate(&cr->lastUpdate_, f_lastUpdate);
 	timevalToDate(&cr->firstUpdate_, f_firstUpdate);
 	f_interface = interfaceName.c_str();
@@ -283,11 +289,12 @@ bool	SQlSaveRecord(CRecord *cr, string interfaceName)
 	} else {
 		ind_contents = -1;
 	}
+	pthread_mutex_lock(&SqlWriteMutex);
 
 	{ ECPGtrans(__LINE__, NULL, "begin transaction");}
-#line 158 "sqlwrite.ec"
+#line 165 "sqlwrite.ec"
 
-	/* contents) */{ ECPGdo(__LINE__, 0, 1, NULL, "insert into usage ( src_ip_serial  , dst_ip_serial  , port_src  , port_dst  , protocol_number  , first_seen  , last_seen  , packets  , volume  , interface  , flushed  ) values (  ? ,  ? ,  ? ,  ? ,  ? ,  ? ,  ? ,  ? ,  ? ,  ? ,  ? )", 
+	/* contents) */{ ECPGdo(__LINE__, 0, 1, NULL, "insert into usage ( src_ip_serial  , dst_ip_serial  , port_src  , port_dst  , protocol_number  , first_seen  , last_seen  , packets  , volume  , interface  , flushed_at  ) values (  ? ,  ? ,  ? ,  ? ,  ? ,  ? ,  ? ,  ? ,  ? ,  ? ,  ? )", 
 	ECPGt_int,&(f_src_ip_serial),(long)1,(long)1,sizeof(int), 
 	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
 	ECPGt_int,&(f_dst_ip_serial),(long)1,(long)1,sizeof(int), 
@@ -308,9 +315,9 @@ bool	SQlSaveRecord(CRecord *cr, string interfaceName)
 	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
 	ECPGt_char,&(f_interface),(long)0,(long)1,1*sizeof(char), 
 	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
-	ECPGt_int,&(f_flushed),(long)1,(long)1,sizeof(int), 
-	ECPGt_int,&(ind_flushed),(long)1,(long)1,sizeof(int), ECPGt_EOIT, ECPGt_EORT);}
-#line 172 "sqlwrite.ec"
+	ECPGt_char,(f_flushed_at),(long)30,(long)1,30*sizeof(char), 
+	ECPGt_int,&(ind_flushed_at),(long)1,(long)1,sizeof(int), ECPGt_EOIT, ECPGt_EORT);}
+#line 179 "sqlwrite.ec"
 
 
 	/*			:f_contents:ind_contents); */
@@ -318,12 +325,17 @@ bool	SQlSaveRecord(CRecord *cr, string interfaceName)
 		syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR), "insert into usage Error %d message '%s'",
 					sqlca.sqlerrm.sqlerrml, sqlca.sqlerrm.sqlerrmc);
 		// delete f_contents;
+		{ ECPGtrans(__LINE__, NULL, "rollback");}
+#line 186 "sqlwrite.ec"
+
+		pthread_mutex_unlock(&SqlWriteMutex);
 		return false;
 	}
 	{ ECPGtrans(__LINE__, NULL, "commit");}
-#line 181 "sqlwrite.ec"
+#line 190 "sqlwrite.ec"
 
 	//delete f_contents;
+	pthread_mutex_unlock(&SqlWriteMutex);
 	return true;
 }
 
@@ -340,32 +352,35 @@ int	SQLGetDNS(string resolvedName, const struct in_addr& ip, const char *mac)
 	 	
 			
 	
-#line 190 "sqlwrite.ec"
+#line 200 "sqlwrite.ec"
  int  f_serial   ;
  
-#line 191 "sqlwrite.ec"
+#line 201 "sqlwrite.ec"
  char  f_hostname [ 512 ]   ;
  
-#line 192 "sqlwrite.ec"
+#line 202 "sqlwrite.ec"
  char  f_ip_address [ 16 ]   ;
  
-#line 194 "sqlwrite.ec"
+#line 204 "sqlwrite.ec"
  const char * q_hostname   ;
  
-#line 195 "sqlwrite.ec"
+#line 205 "sqlwrite.ec"
  const char * q_ip_address   ;
  
-#line 196 "sqlwrite.ec"
+#line 206 "sqlwrite.ec"
  const char * q_mac   ;
  
-#line 197 "sqlwrite.ec"
+#line 207 "sqlwrite.ec"
  int  ind_mac   ;
 /* exec sql end declare section */
-#line 198 "sqlwrite.ec"
+#line 208 "sqlwrite.ec"
+
+	char	ip_char[inet_to_stringSize];
 
 
 	q_hostname = resolvedName.c_str();
-	q_ip_address = inet_to_string(ip).c_str();
+	inet_to_char(ip, ip_char);
+	q_ip_address = ip_char;
 	if (mac != NULL) {
 		ind_mac = 1;
 		q_mac = mac;
@@ -373,9 +388,10 @@ int	SQLGetDNS(string resolvedName, const struct in_addr& ip, const char *mac)
 		ind_mac = -1;
 	}
 
+	pthread_mutex_lock(&SqlWriteMutex);
 	for (;;) {
 		{ ECPGtrans(__LINE__, NULL, "begin transaction");}
-#line 210 "sqlwrite.ec"
+#line 224 "sqlwrite.ec"
 
 		{ ECPGdo(__LINE__, 0, 1, NULL, "select  serial  , hostname  , ip   from dnsentry where hostname  =  ? and ip  =  ?  ", 
 	ECPGt_char,&(q_hostname),(long)0,(long)1,1*sizeof(char), 
@@ -388,7 +404,7 @@ int	SQLGetDNS(string resolvedName, const struct in_addr& ip, const char *mac)
 	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
 	ECPGt_char,(f_ip_address),(long)16,(long)1,16*sizeof(char), 
 	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);}
-#line 214 "sqlwrite.ec"
+#line 228 "sqlwrite.ec"
 
 		
 		if (sqlca.sqlcode == 0) {
@@ -398,7 +414,7 @@ int	SQLGetDNS(string resolvedName, const struct in_addr& ip, const char *mac)
 			syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_INFO), "select from dnsentry Error %d message '%s' ip '%s' host '%s'",
 					sqlca.sqlerrm.sqlerrml, sqlca.sqlerrm.sqlerrmc, q_ip_address, q_hostname);
 			{ ECPGtrans(__LINE__, NULL, "rollback");}
-#line 222 "sqlwrite.ec"
+#line 236 "sqlwrite.ec"
 
 			break;
 		} else {
@@ -409,20 +425,24 @@ int	SQLGetDNS(string resolvedName, const struct in_addr& ip, const char *mac)
 	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, 
 	ECPGt_char,&(q_mac),(long)0,(long)1,1*sizeof(char), 
 	ECPGt_int,&(ind_mac),(long)1,(long)1,sizeof(int), ECPGt_EOIT, ECPGt_EORT);}
-#line 226 "sqlwrite.ec"
+#line 240 "sqlwrite.ec"
 
 
 			if (sqlca.sqlcode != 0) {
 				syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_INFO), "insert into dnsentry Error %d message '%s'",
 						sqlca.sqlerrm.sqlerrml, sqlca.sqlerrm.sqlerrmc);
-				return 0;
+				{ ECPGtrans(__LINE__, NULL, "rollback");}
+#line 245 "sqlwrite.ec"
+
+				break;
 			}
 			// continue on to do select to get serial and return it.
 		}
 		{ ECPGtrans(__LINE__, NULL, "commit");}
-#line 235 "sqlwrite.ec"
+#line 250 "sqlwrite.ec"
 
 	}
+	pthread_mutex_unlock(&SqlWriteMutex);
 	return retCode;
 }
 
